"""GUI testing."""

__author__ = "Hunter McConnell <hunter.mcconnell@usask.ca"

from tkinter import *
from tkinter.ttk import *

import os
import sys
import yaml

from yaml.loader import Loader

from pydtnsim import Node, EpidemicNode
from pydtnsim.community import BubbleKCliqueNode, BubbleLouvainNode
from pydtnsim.community import HCBFKCliqueNode, HCBFLouvainNode


def main(args={"config": "config.yaml"}):
    """Put up a GUI to edit config file."""
    # base window
    root = Tk()
    root.title("PyDtnSim")
    root.geometry("800x400")

    # frames 1-3 divide window into thirds
    frame1 = LabelFrame(
        root, borderwidth=2, text="Frame1", height=200, width=100
    )
    frame1.grid(row=0, column=1)
    frame2 = LabelFrame(root, borderwidth=2, text="Sim", height=200, width=100)
    frame2.grid(row=0, column=0)
    frame3 = LabelFrame(
        root, borderwidth=2, text="Frame3", height=200, width=100
    )
    frame3.grid(row=0, column=2)

    # contains the node checkboxes
    frame_nodes = Frame(frame1)
    frame_nodes.grid(row=0, column=1)

    # node selection
    Label(frame1, text="Nodes:").grid(row=0, column=0)
    NODES = [
        Node,  # direct delivery
        EpidemicNode,
        BubbleKCliqueNode,
        HCBFKCliqueNode,
        BubbleLouvainNode,
        HCBFLouvainNode,
    ]
    node_vars = []

    for (i, node) in zip(range(len(NODES)), NODES):
        v = IntVar()
        node_vars.append(v)
        Checkbutton(frame_nodes, text=node.__name__, variable=v).pack(fill=X)

    Label(frame2, text="Name:").grid(row=0, column=0)
    name_var = StringVar()
    name_var.set("config")
    Entry(frame2, textvariable=name_var).grid(row=0, column=1)

    # contact trace location
    # todo: file dialog?
    # todo: validation?
    Label(frame2, text="Contact file:").grid(row=1, column=0)
    trace_var = StringVar()
    Entry(frame2, textvariable=trace_var).grid(row=1, column=1)

    # output file
    # todo: file dialog?
    # todo: validation?
    Label(frame2, text="Output file:").grid(row=2, column=0)
    out_var = StringVar()
    Entry(frame2, textvariable=out_var).grid(row=2, column=1)

    # batch number
    # todo: validation?
    Label(frame2, text="# of batches:").grid(row=3, column=0)
    batch_var = IntVar(value=1)
    Entry(frame2, textvariable=batch_var).grid(row=3, column=1)

    # seeds
    # todo: disable if batch mode?
    Label(frame2, text="Seeds:").grid(row=4, column=0)
    seeds_var = StringVar()
    Entry(frame2, textvariable=seeds_var).grid(row=4, column=1)

    # epoch
    Label(frame1, text="Epoch (sec):").grid(row=2, column=0)
    epoch_var = IntVar(value=604800)
    Entry(frame1, textvariable=epoch_var).grid(row=2, column=1)

    # tick_rate? or shouldn't this be equal to duty cycles?

    # k for k means clustering
    Label(frame1, text="k:").grid(row=3, column=0)
    k_var = IntVar(value=3)
    Entry(frame1, textvariable=k_var).grid(row=3, column=1)

    # packet creation rate
    Label(frame3, text="Packet Generation Interval (sec):").grid(
        row=1, column=0
    )
    packet_interval_var = IntVar(value=300)
    Entry(frame3, textvariable=packet_interval_var).grid(row=1, column=1)

    # traffic type

    # reads the given config file and allows editing of its values
    def read_config(file_name):
        with open(file_name, "r", newline="") as f:
            param = yaml.load(f, Loader=Loader)
        if "nodes" in param:
            for (node, var) in zip(NODES, node_vars):
                if node in param["nodes"]:
                    var.set(1)
        if "name" in param:
            name_var.set(param["name"])
        if "trace" in param:
            trace_var.set(param["trace"])
        if "output" in param:
            out_var.set(param["output"])
        if "batch" in param:
            batch_var.set(param["batch"])
        if "seeds" in param:
            tmp = ", ".join(str(i) for i in param["seeds"])
            seeds_var.set(tmp)
        if "epoch" in param:
            epoch_var.set(param["epoch"])
        if "packet_interval" in param:
            packet_interval_var.set(param["packet_interval"])

    # read in config file if is exists
    if os.path.exists(args["config"]):
        read_config(args["config"])

    def submit(param):
        """Fill a dict with config values, then dump to file to read later"""
        # todo: validation, must choose at least one node
        param["name"] = name_var.get()
        param["nodes"] = []
        for (node, var) in zip(NODES, node_vars):
            if var.get() == 1:
                param["nodes"].append(node)

        param["trace"] = trace_var.get()
        param["output"] = out_var.get()
        param["batch"] = batch_var.get()
        # transforms the entry string into a list of ints ^.^
        param["seeds"] = [
            int(seed.strip())
            for seed in seeds_var.get().split(",")
            if seed != ""
        ]
        param["epoch"] = epoch_var.get()
        param["packet_interval"] = packet_interval_var.get()

        with open(f"{name_var.get()}.yaml", "w", newline="") as f:
            yaml.dump(param, f)

        msg.set(name_var.get() + " saved")

    def submit_quit(param):
        submit(param)
        root.quit()

    param = {}
    Button(
        root, text="Save sim parameters", command=lambda: submit(param)
    ).grid(row=1, column=0, columnspan=3)
    Button(
        root, text="Save and run sim", command=lambda: submit_quit(param)
    ).grid(row=2, column=0, columnspan=3)
    Button(root, text="Quit", command=root.quit).grid(
        row=3, column=0, columnspan=3
    )
    msg = StringVar(value="   ")
    Label(root, textvariable=msg).grid(row=4, column=0, columnspan=3)

    root.mainloop()

    return param


if __name__ == "__main__":
    sys.exit(main())
